<?php

namespace Teknisa\CUS\Modules\Operacional\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;


class Cus00300CustosDeNaoEstocaveis extends \Zeedhi\Framework\Controller\Report {

	protected $crudService;
	protected $parameterBagColumns;
	protected $connection;
    protected $reportService;
    protected $birtStrategy;
	
	public function __construct($crudService, $parameterBagColumns, $connection, $reportService, $birtStrategy) {
		$this->crudService = $crudService;
		$this->parameterBagColumns = $parameterBagColumns;
		$this->connection = $connection;
        $this->reportService = $reportService;
        $this->birtStrategy = $birtStrategy;
	}
    
    protected $reportMapping = array (
        'cus00300' => 
        array (
          'strategy' => 'BIRT',
          'parameterMapping' => 
          array (
            'NRORG'     => 'P_NRORG',
            'DTMESINIC' => 'P_DTMESINIC',
            'DTMESFINA' => 'P_DTMESFINA',
            'CDFILIAL'  => 'P_CDFILIAL',
            'DTINIC'    => 'P_DTINIC',
            'DTFINA'    => 'P_DTFINA',
            'CDPICTPROD'=> 'P_CDPICTPROD'
          )
        )
    );

	public function duplicarCusto(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();

            $exportReal = false;
            $mesDest = date_create_from_format('d/m/Y', $row['MESDESTINO']);
            $mesProc = date_create_from_format('d/m/Y', $row['DTPROCESSA']);

            if($mesDest <= $mesProc) $exportReal = true;

            if($row['OPERACAO'] == 'S'){
                $sql = "DELETE FROM CUSTNESTO WHERE CDFILIAL = :CDFILIAL AND DTCUSTNESTO = :MESDESTINO";

                $params = array(
                    "CDFILIAL"     => $row["CDFILIAL"],
                    "MESDESTINO"   => $row["MESDESTINO"]
                );

                $std = $this->connection->prepare($sql);
                $std->execute($params);

                $data = $this->getCustosOrigem($row, 'Sobrepor');
            }else $data = $this->getCustosOrigem($row, 'Acrescentar');


            foreach($data as $reg){
                $sql = "INSERT INTO CUSTNESTO
                        (CDFILIAL, DTCUSTNESTO, CDPRODUTO, VRPLANNESTO, VRREALNESTO,  IDLANCNESTO)
                        VALUES
                        (:CDFILIAL, :DTCUSTNESTO, :CDPRODUTO, :VRPLANNESTO, :VRREALNESTO, :IDLANCNESTO)";
            
                $params = array(
                    "CDFILIAL"      => $reg["CDFILIAL"],
                    "DTCUSTNESTO"   => $row["MESDESTINO"],
                    "CDPRODUTO"     => $reg["CDPRODUTO"],
                    "VRPLANNESTO"   => $row['TIPOPLAN'] == 'S'? $reg["VRPLANNESTO"] : null,
                    "VRREALNESTO"   => $row['TIPOREAL'] == 'S' && $exportReal? $reg["VRREALNESTO"] : null,
                    "IDLANCNESTO"   => $reg["IDLANCNESTO"]
                );

                $std = $this->connection->prepare($sql);
                $std->execute($params);
            }
            
            $this->connection->commit();
            $response->addDataset(new DataSet('custoOri', $data));
        } catch (Exception $e) {
            $this->connection->rollBack();
            $response->setError(new Error($e->getMessage(), 500));
            throw $e;
        }
    }

    private function getCustosOrigem($row, $tipo){
        if($tipo == 'Sobrepor'){
            $sql = "SELECT
                    A.CDFILIAL,
                    A.DTCUSTNESTO,
                    A.CDPRODUTO,
                    A.VRPLANNESTO,
                    A.VRREALNESTO,
                    A.IDLANCNESTO,
                    B.NMPRODUTO
                FROM
                    CUSTNESTO A,
                    PRODUTO B
                WHERE
                    A.CDFILIAL    = :P_CDFILIAL    AND
                    A.DTCUSTNESTO = :P_DTCUSTNESTO AND
                    A.CDPRODUTO   = B.CDPRODUTO";
            
            $params = array(
                "P_CDFILIAL"     => $row["CDFILIAL"],
                "P_DTCUSTNESTO"  => $row["MESORIGEM"]
            );
        }else {
            $sql = "SELECT
                    A.CDFILIAL,
                    A.DTCUSTNESTO,
                    A.CDPRODUTO,
                    A.VRPLANNESTO,
                    A.VRREALNESTO,
                    A.IDLANCNESTO,
                    B.NMPRODUTO
                FROM
                    CUSTNESTO A,
                    PRODUTO B
                WHERE
                    A.CDFILIAL    = :P_CDFILIAL    AND
                    A.DTCUSTNESTO = :P_DTCUSTORI AND
                    A.CDPRODUTO   = B.CDPRODUTO AND
                    A.CDPRODUTO NOT IN (SELECT CDPRODUTO 
                                        FROM CUSTNESTO WHERE CDFILIAL = :P_CDFILIAL
                                                        AND DTCUSTNESTO = :P_DTCUSTDEST)";
            
            $params = array(
                "P_CDFILIAL"   => $row["CDFILIAL"],
                "P_DTCUSTORI"  => $row["MESORIGEM"],
                "P_DTCUSTDEST" => $row["MESDESTINO"]
            );
        }

        $data = $this->connection->fetchAll($sql, $params);
        return $data;
    }

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    }
	
}